<div class="form-field">
    <label for="{{ id }}">
        {{ name }}
    </label>
    <div class="dimension-input">
        <input name="dimension-w" type="number"
            id = "{{ id }}-0"
            value="{{ attrs.value[0] }}"
            min="{{ attrs.min or 0}}"
            max="{{ attrs.max or 9999}}">
        <span>x</span>
        <input name="dimension-h" type="number" 
            id = "{{ id }}-1"
            value="{{ attrs.value[1] }}"
            min="{{ attrs.min or 0 }}"
            max="{{ attrs.max or 9999 }}">
    </div>
</div>